package com.brilliance.isc.funds.common.component;

import com.brilliance.isc.bo.Cbb;
import com.brilliance.isc.vo.model.CbbVo;
import com.brilliance.isc.bo.Cnd;
import com.brilliance.isc.bo.Ftd;
import com.brilliance.isc.bo.Ftt;
import com.brilliance.isc.vo.model.FtdVo;
import com.brilliance.isc.vo.model.FttVo;
import com.brilliance.isc.bo.model.*;
import com.brilliance.isc.vo.model.*;
import com.brilliance.isc.common.cbsmod.service.CbsmodService;
import com.brilliance.isc.common.component.PtsptaToolComponent;
import com.brilliance.isc.common.exception.CommonServiceException;
import com.brilliance.isc.common.sysmod.SysmodService;
import com.brilliance.isc.common.transaction.help.TransactionServiceSet;
import com.brilliance.isc.common.trnmod.service.TrnmodService;
import com.brilliance.isc.common.util.StringUtils;
import com.brilliance.isc.common.util.Utils;
import com.brilliance.isc.common.vo.PtsptaVo;

import com.brilliance.isc.mda.dao.*;

import com.brilliance.isc.vo.TrncodVo;
import com.brilliance.mda.runtime.mda.util.Dates;
import com.brilliance.mda.runtime.mda.util.MdaUtils;
import com.brilliance.mda.runtime.mda.util.Strings;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.lang.reflect.Field;
import java.math.BigDecimal;
import java.util.*;

import static com.brilliance.mda.runtime.mda.Constants.NULLSTR;

@Component
public class FtToolComponent {


    //        @Autowired
//        private CpdMapper cpdMapper;
    @Autowired
    private TrnmodService trnmodService;
    @Autowired
    private FtdMapper ftdselMapper;
    @Autowired
    private CbbMapper cbbMapper;
    @Autowired
    private CbsmodService cbsmodService;
    @Autowired
    private PtsptaToolComponent ptsptaToolComponent;
    @Autowired
    private CndMapper cndMapper;
    @Autowired
    private SysmodService sysmodService;

    @Autowired
    private FtdMapper ftdMapper;

    @Autowired
    private FttMapper fttMapper;



    public Ftdgrp getFtdgrpByInr(String inr) {

        Ftd rec = new Ftd();
        rec.setInr(inr);
        rec = ftdMapper.selectByPrimaryKey(rec);
        if (Objects.isNull(rec)) {
            throw new CommonServiceException("主键" + inr + "找不到FTD数据");
        }
        Ftdgrp ftdgrp = new Ftdgrp();
        ftdgrp.reset();
        ftdgrp.setRec(rec);
        Ftt blk = new Ftt();
        blk.setInr(inr);
        blk = fttMapper.selectByPrimaryKey(blk);
        ftdgrp.setBlk(blk);
        //获取cbs金额信息
        Date date = Dates.add(Utils.getMaxDate(), -1);
        cbsmodService.cbsGetBalance(ftdgrp.getCbs().getMax(), "FTD", inr, "MAXSUM", "AMT1", date);
        cbsmodService.cbsGetBalance(ftdgrp.getCbs().getIns(), "FTD", inr, "INSSUM", "AMT1", date);
        cbsmodService.cbsGetBalance(ftdgrp.getCbs().getOpn1(), "FTD", inr, "OPN", "AMT1", date);
        cbsmodService.cbsGetBalance(ftdgrp.getCbs().getOpn2(), "FTD", inr, "OPN2", "AMT1", date);
        //获取ptspta信息
        storePtsToGrp(ftdgrp, inr, "FTD");
        return ftdgrp;
    }

       public void getref(Ftdgrp ftdgrp) {
            //! Set reference number, if not already set (usually called in transaction`s MTABUT\\SAV Event)
            String tbl = NULLSTR;

            switch (ftdgrp.getRec().getFttyp()) {
                case "PT":
                    tbl = "PT";
                    break;
                    /**VP-同存调出
                       VN-同存户转
                    **/
                case "VP":
                    tbl = ftdgrp.getRec().getFttyp();
                    break;
                default:
                    tbl = "FT";
            }
            if (MdaUtils.isEmpty(ftdgrp.getRec().getOwnref())) {

                ftdgrp.getRec().setOwnref(sysmodService.getRef(tbl, 5));
            }

        }

    public void getCndInfo(Cpdgrp cpdgrp, String objInr, String objType) {
        List<Cnd> cnds = cndMapper.selectByObjtypAndObjinr(objType, objInr);
        if (!MdaUtils.isEmpty(cnds)) {
            cpdgrp.setPyebchkey(cnds.get(0).getPyebchkey());
            cpdgrp.setPyebchnam(cnds.get(0).getPyebchnam());
        }
    }

    public void storePtsToGrp(Object grpObject, String objInr, String objType) {
        //找ptspta数据
        Map<String, PtsptaVo> maps = ptsptaToolComponent.getPtsptaList(objInr, objType);
        if (maps.size() > 0) {
            maps.keySet().forEach(key -> {
                PtsptaVo ptsptaVo = maps.get(key);
                if (key.equalsIgnoreCase("INT")) {
                    key = "tmpint";
                }
                Utils.setFieldValue(grpObject, key.toLowerCase(), ptsptaVo);
            });
        }
    }

    public void collectCbsMap(TransactionServiceSet set, String prefix, Ftacbs cbs, String... cbsNames) {
        Map<String, Cbb> cbsMap = set.getCbsMap();
        if (cbsMap == null) {
            cbsMap = new HashMap<>();
        }
        for (String name : cbsNames) {
            Cbb cbb = Utils.getFieldValue(cbs, name);
            if (cbb != null) {
                cbsMap.put(prefix.toUpperCase() + name.toUpperCase(), cbb);
            }
        }
        set.setCbsMap(cbsMap);
    }


    public void collectPtsList(TransactionServiceSet set, Ftdgrp ftdgrp, String... ptsNames) {
        if (ptsNames == null) {
            //找所有的ptspta对象
            Field[] fields = ftdgrp.getClass().getDeclaredFields();
            for (Field field : fields) {
                field.setAccessible(true);
                if (PtsptaVo.class.equals(field.getType())) {
                    PtsptaVo ptspta = Utils.getFieldValue(ftdgrp, field.getName());
                    if (ptspta != null) {
                        set.addPtspta(transferFrom(ptspta));
                    }
                }
            }
        } else {
            for (String name : ptsNames) {
                PtsptaVo ptspta = Utils.getFieldValue(ftdgrp, name);
                if (ptspta != null && ptspta.getPts() != null && StringUtils.isNotEmpty(ptspta.getPts().getRol())) {
                    set.addPtspta(transferFrom(ptspta));
                }
            }
        }
    }



    /**
     * 根据STL结构的ptspta转化成抽象组件的ptspta，结构不一样
     *
     * @param ptsptas
     * @return List<com.brilliance.common.vo.PtsptaVo>
     */
    protected PtsptaVo transferFrom(PtsptaVo ptsptas) {
        PtsptaVo ptspta = ptsptas;
        ptspta.setRol(ptsptas.getPts().getRol());
        ptspta.setPtsinr(ptsptas.getPts().getInr());
        ptspta.setNam(ptsptas.getPts().getNam());
        ptspta.setPtyinr(ptsptas.getPts().getPtyinr());
        ptspta.setPtainr(ptsptas.getPts().getPtainr());
        ptspta.setExtkey(ptsptas.getPts().getExtkey());
        ptspta.setDftdsp(ptsptas.getPts().getDftdsp());
        ptspta.setDftcur(ptsptas.getPts().getDftcur());
        ptspta.setDftact(ptsptas.getPts().getDftact());
        ptspta.setDftfeecur(ptsptas.getPts().getDftfeecur());
        ptspta.setDftactptainr(ptsptas.getPts().getDftactptainr());
        ptspta.setDftselrol(ptsptas.getPts().getDftsetrol());
        ptspta.setGlggrpflg(ptsptas.getPts().getGlggrpflg());
        ptspta.setRef(ptsptas.getPts().getRef());
        ptspta.setPtytyp(ptsptas.getPtytyp());
        ptspta.setPtyextkey(ptsptas.getPtyextkey());
        ptspta.setAdrblk(ptsptas.getPts().getAdrblk());
        return ptspta;
    }
        public void sav(Ftdgrp ftdgrp) {

        trnmodService.trnStoreContract(ftdgrp);
        }

    public void setObjNam(String objref, String nam) {
        //Setting OBJNAM. If the setting of OBJREF is not standard, here you can change it.
//        ctx.absGet(Sysmod.class).setObjnam(NULLSTR);
//        ctx.absGet(Sysmod.class).setObjnam(objref + "/" + nam);


    }

    public String getFtdInrByOwnref(String ownref) {
        Ftd ftd= ftdMapper.selectByOwnref(ownref);

        if (ftd !=null && StringUtils.isNotEmpty(ftd.getInr())) {
            return ftd.getInr();
        } else {
            return null;
        }

    }

    public static void isFTTrnAllowed(Ftdgrp ftdgrp, TrncodVo trncodVo)
    {

        ftdgrp.reset();
        String trnNam = trncodVo.getInifrm().toUpperCase();
        String errOpnY ="Selected funds transfer is already opened.";
        String errOpnN ="No funds transfer selected.";
        String errAccN ="There is no amount in transit.";
        String errCnfY ="This funds transfer has beed confirmed.";
        String errCnfN ="This funds transfer has not yet confirmed.";
        String errClsY ="This funds transfer has been closed.";
        String errMatY ="This funds transfer has no open balance.";
        String errZJB = "此交易只能资金部经办！";
        String errYYB = "此交易只能营运部经办！";




        if ("FTTFCM".equals(trnNam)||"FTTPCM".equals(trnNam)) {

            if (!StringUtils.isEmpty(ftdgrp.getRec().getInr())){
                trncodVo.setDisResult(errOpnY);
                trncodVo.setIsAllowed("N");
            }
        }

        if ("FTTFMT".equals(trnNam)) {
            if (StringUtils.isEmpty(ftdgrp.getRec().getInr())){
                trncodVo.setDisResult(errOpnN);
                trncodVo.setIsAllowed("N");
            }else if( MdaUtils.isEmpty(ftdgrp.getRec().getCnfdat())){
                trncodVo.setDisResult(errCnfY);
                trncodVo.setIsAllowed("N");
            }

            if( ! MdaUtils.isEmpty(ftdgrp.getRec().getClsdat()) ) {
                trncodVo.setDisResult(errClsY);
                trncodVo.setIsAllowed("N");
            }
            else if( MdaUtils.isEmpty(ftdgrp.getCbs().getOpn1().getAmt()) ) {
                trncodVo.setDisResult(errMatY);
                trncodVo.setIsAllowed("N");
            }
            if( MdaUtils.compareTo(ftdgrp.getRec().getFttyp(),"PT") == 0 )
            {
                trncodVo.setDisResult("");
                trncodVo.setIsAllowed("N");
            }
//            if( ! Platform.checkUsrBranch("2") ) {     // 资金调拨到期交易只能营运部做
//                trncodVo.setDisResult(errYYB);
//                trncodVo.setIsAllowed("N");
//            }

        }

        if ("FTTFCN".equals(trnNam)){
            if( MdaUtils.isEmpty(ftdgrp.getRec().getInr()) )
            {
                trncodVo.setDisResult(errOpnN);
                trncodVo.setIsAllowed("N");
            }
            else
            {
                if( MdaUtils.compareTo(ftdgrp.getCbs().getOpn2().getAmt(),0)<=0 )
                {
                    trncodVo.setDisResult(errAccN);
                    trncodVo.setIsAllowed("N");
                }
            }
            if( MdaUtils.compareTo(ftdgrp.getRec().getFttyp(),"PT") == 0 )
            {
                trncodVo.setDisResult("");
                trncodVo.setIsAllowed("N");
            }
//            if( ! Platform.checkUsrBranch("2") )
//            {     // 资金调拨销账交易只能营运部做
//
//                trncodVo.setDisResult(errYYB);
//                trncodVo.setIsAllowed("N");
//            }
        }

        if ("FTTFCN".equals(trnNam)){
            if( MdaUtils.isEmpty(ftdgrp.getRec().getInr()) )
            {
                trncodVo.setDisResult(errOpnN);
                trncodVo.setIsAllowed("N");
            }
            else
            {
                if( MdaUtils.compareTo(ftdgrp.getCbs().getOpn2().getAmt(),0)<=0 )
                {
                    trncodVo.setDisResult(errAccN);
                    trncodVo.setIsAllowed("N");
                }
            }
            if( MdaUtils.compareTo(ftdgrp.getRec().getFttyp(),"PT") != 0 )
            {
                trncodVo.setDisResult("");
                trncodVo.setIsAllowed("N");
            }
//            if( ! Platform.checkUsrBranch("2") )
//            {     // 资金调拨销账交易只能营运部做
//                trncodVo.setDisResult(errYYB);
//                trncodVo.setIsAllowed("N");
//            }

        }

        if ("FTTFCN".equals(trnNam)){
            if( MdaUtils.isEmpty(ftdgrp.getRec().getInr()) )
            {
                trncodVo.setDisResult(errOpnN);
                trncodVo.setIsAllowed("N");
            }
            else
            {
                if( MdaUtils.compareTo(ftdgrp.getCbs().getOpn2().getAmt(),0)<=0 )
                {
                    trncodVo.setDisResult(errAccN);
                    trncodVo.setIsAllowed("N");
                }
            }
            if( MdaUtils.compareTo(ftdgrp.getRec().getFttyp(),"PT") != 0 )
            {
                trncodVo.setDisResult("");
                trncodVo.setIsAllowed("N");
            }
//            if( ! Platform.checkUsrBranch("2") )
//            {     // 资金调费用交易交易只能营运部做
//
//                trncodVo.setDisResult(errYYB);
//                trncodVo.setIsAllowed("N");
//            }
        }

        if ("FTTFCN".equals(trnNam)){
            if( MdaUtils.isEmpty(ftdgrp.getRec().getInr()) )
            {
                trncodVo.setDisResult(errOpnN);
                trncodVo.setIsAllowed("N");
            }
        }

        if ("FTTFCN".equals(trnNam)){
            if( MdaUtils.isEmpty(ftdgrp.getRec().getInr()) )
            {
                trncodVo.setDisResult(errOpnN);
                trncodVo.setIsAllowed("N");
            }
            if( MdaUtils.compareTo(ftdgrp.getRec().getFttyp(),"PT") == 0 )
            {
                trncodVo.setDisResult("");
                trncodVo.setIsAllowed("N");
            }
        }


        if(MdaUtils.isEmpty(trnNam)) {
            String msgTxt = "The selected transaction cannot be started under the selected F/T.";
            trncodVo.setDisResult(msgTxt);
            trncodVo.setIsAllowed("N");
        }
        if(Strings.isEmpty(trncodVo.getDisResult())){
            trncodVo.setIsAllowed("Y");
        }

    }

}


