package com.brilliance.isc.funds.common.component;

import com.brilliance.isc.bo.Cbb;
import com.brilliance.isc.bo.Fxd;
import com.brilliance.isc.bo.Fxt;
import com.brilliance.isc.bo.model.Fxacbs;
import com.brilliance.isc.bo.model.Fxdgrp;
import com.brilliance.isc.common.cbsmod.service.CbsmodService;
import com.brilliance.isc.common.component.PtsptaToolComponent;
import com.brilliance.isc.common.context.SettleContext;
import com.brilliance.isc.common.exception.CommonServiceException;
import com.brilliance.isc.common.sysmod.SysmodService;
import com.brilliance.isc.common.transaction.help.TransactionServiceSet;
import com.brilliance.isc.common.trnmod.service.TrnmodService;
import com.brilliance.isc.common.util.I18nUtil;
import com.brilliance.isc.common.util.StringUtils;
import com.brilliance.isc.common.util.Utils;
import com.brilliance.isc.common.vo.PtsptaVo;
import com.brilliance.isc.mda.dao.FxdMapper;
import com.brilliance.isc.mda.dao.FxtMapper;
import com.brilliance.isc.mda.dao.PublicMapper;
import com.brilliance.isc.vo.TrncodVo;
import com.brilliance.mda.runtime.mda.util.Dates;
import com.brilliance.mda.runtime.mda.util.MdaUtils;
import com.brilliance.mda.runtime.mda.util.Strings;
import com.google.common.collect.ImmutableBiMap;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.lang.reflect.Field;
import java.math.BigDecimal;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

import static com.brilliance.mda.runtime.mda.Constants.CR;
import static com.brilliance.mda.runtime.mda.Constants.NULLSTR;

@Component
public class FxToolComponent {

    @Resource
    private FxdMapper fxdMapper;

    @Resource
    private FxtMapper fxtMapper;

    @Resource
    private PtsptaToolComponent ptsptaToolComponent;

    @Resource
    private CbsmodService cbsmodService;

    @Resource
    private SysmodService sysmodService;

    @Resource
    private TrnmodService trnmodService;

    @Resource
    private PublicMapper publicMapper;

    public Fxdgrp getFxdgrpByInr(String inr) {
        Fxd rec = new Fxd();
        rec.setInr(inr);
        rec = fxdMapper.selectByPrimaryKey(rec);
        if (Objects.isNull(rec)) {
            throw new CommonServiceException("主键" + inr + "找不到FXD数据");
        }
        Fxdgrp fxdgrp = new Fxdgrp();
        fxdgrp.reset();
        fxdgrp.setRec(rec);
        Fxt blk = new Fxt();
        blk.setInr(inr);
        blk = fxtMapper.selectByPrimaryKey(blk);
        fxdgrp.setBlk(blk);
        //获取cbs金额信息
        Date date = Dates.add(Utils.getMaxDate(), -1);
        cbsmodService.cbsGetBalance(fxdgrp.getCbs().getMax(), "FXD", inr, "MAXSUM", "AMT1", date);
        cbsmodService.cbsGetBalance(fxdgrp.getCbs().getNom1(), "FXD", inr, "NOMSUM", "AMT1", date);
        cbsmodService.cbsGetBalance(fxdgrp.getCbs().getOpn1(), "FXD", inr, "OPN", "AMT1", date);
        cbsmodService.cbsGetBalance(fxdgrp.getCbs().getOpn2(), "FXD", inr, "OPN2", "AMT1", date);
        //获取ptspta信息
        storePtsToGrp(fxdgrp, inr, "FXD");
        return fxdgrp;
    }


    public void storePtsToGrp(Object grpObject, String objInr, String objType) {
        //找ptspta数据
        Map<String, PtsptaVo> maps = ptsptaToolComponent.getPtsptaList(objInr, objType);
        if (maps.size() > 0) {
            maps.keySet().forEach(key -> {
                PtsptaVo ptsptaVo = maps.get(key);
                if (key.equalsIgnoreCase("INT")) {
                    key = "tmpint";
                }
                Utils.setFieldValue(grpObject, key.toLowerCase(), ptsptaVo);
            });
        }
    }

    public void collectCbsMap(TransactionServiceSet set, String prefix, Fxacbs cbs, String... cbsNames) {
        Map<String, Cbb> cbsMap = set.getCbsMap();
        if (cbsMap == null) {
            cbsMap = new HashMap<>();
        }
        for (String name : cbsNames) {
            Cbb cbb = Utils.getFieldValue(cbs, name);
            if (cbb != null) {
                cbsMap.put(prefix.toUpperCase() + name.toUpperCase(), cbb);
            }
        }
        set.setCbsMap(cbsMap);
    }


    public void collectPtsList(TransactionServiceSet set, Object fxdgrp, String... ptsNames) {
        if (ptsNames == null) {
            //找所有的ptspta对象
            Field[] fields = fxdgrp.getClass().getDeclaredFields();
            for (Field field : fields) {
                field.setAccessible(true);
                if (PtsptaVo.class.equals(field.getType())) {
                    PtsptaVo ptspta = Utils.getFieldValue(fxdgrp, field.getName());
                    if (ptspta != null) {
                        set.addPtspta(transferFrom(ptspta));
                    }
                }
            }
        } else {
            for (String name : ptsNames) {
                PtsptaVo ptspta = Utils.getFieldValue(fxdgrp, name);
                if (ptspta != null && ptspta.getPts() != null && StringUtils.isNotEmpty(ptspta.getPts().getRol())) {
                    set.addPtspta(transferFrom(ptspta));
                }
            }
        }
    }


    /**
     * 根据STL结构的ptspta转化成抽象组件的ptspta，结构不一样
     *
     * @param ptsptas
     * @return List<com.brilliance.common.vo.PtsptaVo>
     */
    protected PtsptaVo transferFrom(PtsptaVo ptsptas) {
        PtsptaVo ptspta = ptsptas;
        ptspta.setRol(ptsptas.getPts().getRol());
        ptspta.setPtsinr(ptsptas.getPts().getInr());
        ptspta.setNam(ptsptas.getPts().getNam());
        ptspta.setPtyinr(ptsptas.getPts().getPtyinr());
        ptspta.setPtainr(ptsptas.getPts().getPtainr());
        ptspta.setExtkey(ptsptas.getPts().getExtkey());
        ptspta.setDftdsp(ptsptas.getPts().getDftdsp());
        ptspta.setDftcur(ptsptas.getPts().getDftcur());
        ptspta.setDftact(ptsptas.getPts().getDftact());
        ptspta.setDftfeecur(ptsptas.getPts().getDftfeecur());
        ptspta.setDftactptainr(ptsptas.getPts().getDftactptainr());
        ptspta.setDftselrol(ptsptas.getPts().getDftsetrol());
        ptspta.setGlggrpflg(ptsptas.getPts().getGlggrpflg());
        ptspta.setRef(ptsptas.getPts().getRef());
        ptspta.setPtytyp(ptsptas.getPtytyp());
        ptspta.setPtyextkey(ptsptas.getPtyextkey());
        ptspta.setAdrblk(ptsptas.getPts().getAdrblk());
        return ptspta;
    }


    /**
     * source:fxtmod.0000.script
     */
    public void getref(Fxdgrp fxdgrp) {
        //! Set reference number, if not already set (usually called in transaction`s MTABUT\\SAV Event)
        String tbl = NULLSTR;
        int len = 0;
        switch (fxdgrp.getRec().getFxtyp()) {
            case "LS":
            case "LB":
                tbl = "FX";
                len = 4;
                break;
            case "XS":
            case "XB":
                tbl = "DX";
                len = 4;
                break;
            case "US":
                tbl = "S" + Strings.mid(fxdgrp.getCbs().getMax().getCur(), 1, 1);
                len = 4;
                break;
            case "UB":
                tbl = "B" + Strings.mid(fxdgrp.getCbs().getMax().getCur(), 1, 1);
                len = 4;
                break;
            default:
                tbl = "J" + Strings.mid(fxdgrp.getCbs().getMax().getCur(), 1, 1);
                len = 4;
        }
        if (MdaUtils.isEmpty(fxdgrp.getRec().getOwnref())) {

            fxdgrp.getRec().setOwnref(sysmodService.getRef(tbl, len));
        }

    }

    /**
     * source:fxtmod.0001.script
     */
    public void sav(Fxdgrp fxdgrp, BigDecimal cshpct) {
        // saves contract including texts. CBS resp. of transaction
        String frm = SettleContext.getTransName();
        switch (frm) {
            case "FXTSUS":
            case "FXTSUB":
            case "FXTSST":
            case "FXTSCN":
                //Datas.getContentNumeric(this, "\\LIAALL\\LIACCV\\CSHPCT")
                fxdgrp.getRec().setCshpct(cshpct);
                break;
        }
        trnmodService.trnStoreContract(fxdgrp);
    }

    /**
     * source:fxtmod.0002.script
     */
    public void setObjNam(String objref, String nam) {
        //Setting OBJNAM. If the setting of OBJREF is not standard, here you can change it.
//        ctx.absGet(Sysmod.class).setObjnam(NULLSTR);
//        ctx.absGet(Sysmod.class).setObjnam(objref + "/" + nam);

    }


    public String getFxdInrByOwnref(String ownref) {
        Fxd fxd = fxdMapper.selectByOwnref(ownref);

        if (fxd != null && StringUtils.isNotEmpty(fxd.getInr())) {
            return fxd.getInr();
        } else {
            return null;
        }

    }


    public static void isFXTrnAllowed(Fxdgrp fxdgrp, TrncodVo trncodVo) {

        fxdgrp.reset();
        String trnNam = trncodVo.getInifrm().toUpperCase();
        String errOpnY = "Selected foreign exchange is already opened.";
        String errOpnN = "No foreign exchange selected.";
        String errClsY = "Selected foreign exchange has been closed.";
        String errQuoY = "Selected foreign exchange has been quoted.";
        String errQuoN = "Selected foreign exchange has not quoted yet.";
        String errSigY = "Sight exchange cannot settle at maturity.";
        String errCnfY = "Selected foreign exchange has been confirmed.";
        String errCnfN = "Selected foreign exchange has not been confirmed.";
        String errAccN = "There is no amount in transit.";
        String errZJB = "此交易只能资金部经办！";
        String errYYB = "此交易只能营运部经办！";
        String errHO = "Only Headoffice can quote.";
        String errRel = "The former registration transaction(FXTSOP/FXTBOP) not released yet.";
        String errSetY = "Selected foreign exchange has been settled.";
        String errLB = "";


        if ("FXTLOP".equals(trnNam) || "FXTFOP".equals(trnNam) || "FXTDEP".equals(trnNam)) {

            if (!StringUtils.isEmpty(fxdgrp.getRec().getInr())) {
                trncodVo.setDisResult(errOpnY);
                trncodVo.setIsAllowed("N");
            }
        }

        if ("FXTLCM".equals(trnNam) || "FXTFCM".equals(trnNam) || "FXTDEM".equals(trnNam)) {

            if (StringUtils.isEmpty(fxdgrp.getRec().getInr())) {
                trncodVo.setDisResult(errOpnN);
                trncodVo.setIsAllowed("N");
            } else if (Objects.nonNull(fxdgrp.getRec().getCnfdat())) {
                trncodVo.setDisResult(errCnfY);
                trncodVo.setIsAllowed("N");
            }

            if (Objects.nonNull(fxdgrp.getRec().getClsdat())) {
                trncodVo.setDisResult(errClsY);
                trncodVo.setIsAllowed("N");
            }

            //todo
//            if (! CheckUsrBranch( "2" )) {
//                //结售汇平盘确认交易只能营运部做
//                trncodVo.setDisResult(errYYB);
//                trncodVo.setIsAllowed("N");
//            }

        }


        if ("FXTLCN".equals(trnNam) || "FXTSUC".equals(trnNam) || "FXTFCN".equals(trnNam) || "FXTDEN".equals(trnNam)) {
            if (StringUtils.isEmpty(fxdgrp.getRec().getInr())) {
                trncodVo.setDisResult(errOpnN);
                trncodVo.setIsAllowed("N");
            } else if (fxdgrp.getCbs().getOpn2().getAmt().compareTo(BigDecimal.valueOf(0)) <= 0) {
                trncodVo.setDisResult(errAccN);
                trncodVo.setIsAllowed("N");
            }

            if (Objects.nonNull(fxdgrp.getRec().getClsdat())) {
                trncodVo.setDisResult(errClsY);
                trncodVo.setIsAllowed("N");
            }

            if (Objects.isNull(fxdgrp.getRec().getCnfdat())) {
                trncodVo.setDisResult(errCnfN);
                trncodVo.setIsAllowed("N");
            }

        }


        if (MdaUtils.isEmpty(trnNam)) {
            String msgTxt = "The selected transaction cannot be started under the selected F/X.";
            trncodVo.setDisResult(msgTxt);
            trncodVo.setIsAllowed("N");
        }
        if (Strings.isEmpty(trncodVo.getDisResult())) {
            trncodVo.setIsAllowed("Y");
        }

    }


    /**
     * source:fxdlod.0011.script
     */
    public void isFXTrnAllowed2(Fxdgrp argFxdGrp, TrncodVo trncodVo) {
        //! Defines, which transaction can be executed based on the passed contract.
        //! The passed contract has to be loaded completely.
        //! If the Transaction is allowed ArgRes returns an empty value.
        //! otherwise ArgRes will hold an Errormessage.
        // normally the decision, if a transaction is allowed, is based on the
        // following 3 information:
        // - contract loaded (ArgLidGrp\\rec\\inr not empty)
        // - contract opened (ArgLidGrp\\rec\\opndat not empty)
        // - contract not closed (ArgLidGrp\\rec\\clsdat empty)
        // Initially set the return values to allowed and no errormessage.
        argFxdGrp.reset();
        String trnNam = trncodVo.getInifrm();
        boolean res = true;
        // Initialize the variables used to assemble errormessage
        // Attention: MagTxt maximum size is 5 rows with 80 digits per row
        String msgTxt = "";
        String cr = "";
        //Errortexts
        String errOpnY = I18nUtil.getMessage("funds_inffxd_000000");
        String errOpnN = I18nUtil.getMessage("funds_inffxd_000001");
        String errClsY = I18nUtil.getMessage("funds_inffxd_000002");
        String errQuoY = I18nUtil.getMessage("funds_inffxd_000003");
        String errQuoN = I18nUtil.getMessage("funds_inffxd_000004");
        String errSigY = I18nUtil.getMessage("funds_inffxd_000005");
        String errCnfY = I18nUtil.getMessage("funds_inffxd_000006");
        String errCnfN = I18nUtil.getMessage("funds_inffxd_000007");
        String errAccN = I18nUtil.getMessage("funds_inffxd_000008");
        String errZJB = I18nUtil.getMessage("funds_inffxd_000009");
        String errYYB = I18nUtil.getMessage("funds_inffxd_000010");
        String errHO = I18nUtil.getMessage("funds_inffxd_000011");
        String errRel = I18nUtil.getMessage("funds_inffxd_000012");
        String errSetY = I18nUtil.getMessage("funds_inffxd_000013");
        String errLB = "";
        // All transactions should be listed in the select case.
        // A condition might inhibit the usage of the contract with the specified
        // transaction by either setting $Res to false or by setting an errormessage
        // into ArgRes.
        switch (Strings.toUpper(trnNam)) {
            case "FXTSOP":
            case "FXTBOP":
            case "FXTSSS":
            case "FXTSSB":
            case "FXTSUS":
            case "FXTSUB":
                if (!MdaUtils.isEmpty(argFxdGrp.getRec().getInr())) {
                    msgTxt = msgTxt + cr + errOpnY;
                    cr = CR;
                    res = false;
                }
                /**
                 if CheckUsrBranch( "1" ) then     # 结售汇登记交易只能营运部做
                 $MsgTxt = $MsgTxt + $CR + $ErrYYB
                 $CR = CR
                 $res = FALSE
                 endif
                 **/
                break;
            case "FXTSQO":
                if (MdaUtils.isEmpty(argFxdGrp.getRec().getInr())) {
                    msgTxt = msgTxt + cr + errOpnN;
                    cr = CR;
                    res = false;
                }
                if (!MdaUtils.isEmpty(argFxdGrp.getRec().getClsdat())) {
                    msgTxt = msgTxt + cr + errClsY;
                    cr = CR;
                    res = false;
                }
                if (!MdaUtils.isEmpty(argFxdGrp.getRec().getRat())) {
                    msgTxt = msgTxt + cr + errQuoY;
                    cr = CR;
                    res = false;
                }
                /**
                 if not CheckUsrBranch( "1" ) then     # 结售汇报价交易只能资金部做
                 $MsgTxt = $MsgTxt + $CR + $ErrZJB
                 $CR = CR
                 $res = FALSE
                 endif
                 if \\SYSMOD\\BCH\\LEV <> "0" then
                 $MsgTxt = $MsgTxt + $CR + $ErrHO
                 $CR = CR
                 $res = FALSE
                 endif
                 **/
                if (!checkRelStatus(argFxdGrp.getRec().getInr())) {
                    msgTxt = msgTxt + cr + errRel;
                    cr = CR;
                    res = false;
                }
                break;
            case "FXTSST":
            case "FXTSCN":
                if (MdaUtils.isEmpty(argFxdGrp.getRec().getInr())) {
                    msgTxt = msgTxt + cr + errOpnN;
                    cr = CR;
                    res = false;
                } else {
                    if (MdaUtils.compareTo(Strings.mid(argFxdGrp.getRec().getFxtyp(), 1, 1), "S") == 0) {
                        msgTxt = msgTxt + cr + errSigY;
                        cr = CR;
                        res = false;
                    }
                }
                if (!MdaUtils.isEmpty(argFxdGrp.getRec().getClsdat())) {
                    msgTxt = msgTxt + cr + errClsY;
                    cr = CR;
                    res = false;
                }
                if (MdaUtils.isEmpty(argFxdGrp.getRec().getRat())) {
                    msgTxt = msgTxt + cr + errQuoN;
                    cr = CR;
                    res = false;
                }
                if (!MdaUtils.isEmpty(argFxdGrp.getRec().getCnfdat())) {
                    msgTxt = msgTxt + cr + errSetY;
                    cr = CR;
                    res = false;
                }
                /**
                 if not CheckUsrBranch( "2" ) then     # 结售汇到期、撤销交易只能营运部做
                 $MsgTxt = $MsgTxt + $CR + $ErrYYB
                 $CR = CR
                 $res = FALSE
                 endif
                 **/
                break;
            case "FXTEOP":
                if (!MdaUtils.isEmpty(argFxdGrp.getRec().getInr())) {
                    msgTxt = msgTxt + cr + errOpnY;
                    cr = CR;
                    res = false;
                }
                /**
                 if CheckUsrBranch( "1" ) then     # 代客外汇买卖登记交易只能营运部做
                 $MsgTxt = $MsgTxt + $CR + $ErrYYB
                 $CR = CR
                 $res = FALSE
                 endif
                 **/
                break;
            case "FXTEQO":
                /**
                 if not IsEmpty( ArgFxdGrp\\Rec\\Inr ) then
                 $MsgTxt = $MsgTxt + $CR + $ErrOpnY
                 $CR = CR
                 $res = FALSE
                 endif
                 **/
                if (MdaUtils.isEmpty(argFxdGrp.getRec().getInr())) {
                    msgTxt = msgTxt + cr + errOpnN;
                    cr = CR;
                    res = false;
                }
                if (!MdaUtils.isEmpty(argFxdGrp.getRec().getClsdat())) {
                    msgTxt = msgTxt + cr + errClsY;
                    cr = CR;
                    res = false;
                }
                if (!MdaUtils.isEmpty(argFxdGrp.getRec().getRat())) {
                    msgTxt = msgTxt + cr + errQuoY;
                    cr = CR;
                    res = false;
                }
                if (!checkUsrBranch("1")) {   // 代客外汇买卖报价交易只能资金部做
                    msgTxt = msgTxt + cr + errZJB;
                    cr = CR;
                    res = false;
                }
                if (!checkRelStatus(argFxdGrp.getRec().getInr())) {
                    msgTxt = msgTxt + cr + errRel;
                    cr = CR;
                    res = false;
                }
                break;
            case "FXTLEP":
                if (MdaUtils.isEmpty(argFxdGrp.getRec().getInr())) {
                    msgTxt = msgTxt + cr + errOpnN;
                    cr = CR;
                    res = false;
                } else {
                    if (MdaUtils.compareTo(argFxdGrp.getCbs().getOpn1().getAmt(), 0) == 0) {
                        msgTxt = msgTxt + cr + errLB;
                        cr = CR;
                        res = false;
                    }
                }
                if (!MdaUtils.isEmpty(argFxdGrp.getRec().getClsdat())) {
                    msgTxt = msgTxt + cr + errClsY;
                    cr = CR;
                    res = false;
                }
                if (!checkRelStatus(argFxdGrp.getRec().getInr())) {
                    msgTxt = msgTxt + cr + errRel;
                    cr = CR;
                    res = false;
                }
                break;
            case "FXTLOP":
            case "FXTFOP":
            case "FXTDEP":
                if (!MdaUtils.isEmpty(argFxdGrp.getRec().getInr())) {
                    msgTxt = msgTxt + cr + errOpnY;
                    cr = CR;
                    res = false;
                }
                break;
            case "FXTLCM":
            case "FXTFCM":
            case "FXTDEM":
                if (MdaUtils.isEmpty(argFxdGrp.getRec().getInr())) {
                    msgTxt = msgTxt + cr + errOpnN;
                    cr = CR;
                    res = false;
                } else {
                    if (!MdaUtils.isEmpty(argFxdGrp.getRec().getCnfdat())) {
                        msgTxt = msgTxt + cr + errCnfY;
                        cr = CR;
                        res = false;
                    }
                }
                if (!MdaUtils.isEmpty(argFxdGrp.getRec().getClsdat())) {
                    msgTxt = msgTxt + cr + errClsY;
                    cr = CR;
                    res = false;
                }
                if (!checkUsrBranch("2")) {     // 结售汇平盘确认交易只能营运部做
                    msgTxt = msgTxt + cr + errYYB;
                    cr = CR;
                    res = false;
                }
                break;
            case "FXTLCN":
            case "FXTSUC":
            case "FXTFCN":
            case "FXTDEN":
                if (MdaUtils.isEmpty(argFxdGrp.getRec().getInr())) {
                    msgTxt = msgTxt + cr + errOpnN;
                    cr = CR;
                    res = false;
                } else {
                    if (MdaUtils.compareTo(argFxdGrp.getCbs().getOpn2().getAmt(), 0) <= 0) {
                        msgTxt = msgTxt + cr + errAccN;
                        cr = CR;
                        res = false;
                    }
                    /**
                     if ( ArgFxdGrp\\Rec\\FxTyp == "LB" and ArgFxdGrp\\Cbs\\Opn1\\Amt > 0 ) or ArgFxdGrp\\Cbs\\Opn2\\Amt <= 0 then
                     $MsgTxt = $MsgTxt + $CR + $ErrLB
                     $CR = CR
                     $res = FALSE
                     endif
                     **/
                }
                if (!MdaUtils.isEmpty(argFxdGrp.getRec().getClsdat())) {
                    msgTxt = msgTxt + cr + errClsY;
                    cr = CR;
                    res = false;
                }
                if (MdaUtils.isEmpty(argFxdGrp.getRec().getCnfdat())) {
                    msgTxt = msgTxt + cr + errCnfN;
                    cr = CR;
                    res = false;
                }
                /**
                 if not CheckUsrBranch( "2" ) then     # 结售汇平盘销账交易只能营运部做
                 $MsgTxt = $MsgTxt + $CR + $ErrYYB
                 $CR = CR
                 $res = FALSE
                 endif
                 **/
                //代客外汇买卖平盘
                break;
            case "FXTLER":
                if (!MdaUtils.isEmpty(argFxdGrp.getRec().getInr())) {
                    msgTxt = msgTxt + cr + errOpnY;
                    cr = CR;
                    res = false;
                }
                break;
            case "FXTLEM":
                if (MdaUtils.isEmpty(argFxdGrp.getRec().getInr())) {
                    msgTxt = msgTxt + cr + errOpnN;
                    cr = CR;
                    res = false;
                } else {
                    if (MdaUtils.compareTo(argFxdGrp.getCbs().getOpn2().getAmt(), 0) == 0) {
                        msgTxt = msgTxt + cr + errLB;
                        cr = CR;
                        res = false;
                    }
                }
                if (!MdaUtils.isEmpty(argFxdGrp.getRec().getClsdat())) {
                    msgTxt = msgTxt + cr + errClsY;
                    cr = CR;
                    res = false;
                }
                if (!checkRelStatus(argFxdGrp.getRec().getInr())) {
                    msgTxt = msgTxt + cr + errRel;
                    cr = CR;
                    res = false;
                }
                break;
            case "FXTLEN":
                if (MdaUtils.isEmpty(argFxdGrp.getRec().getInr())) {
                    msgTxt = msgTxt + cr + errOpnN;
                    cr = CR;
                    res = false;
                } else {
                    //if ( ArgFxdGrp\\Rec\\FxTyp = "LX" and ArgFxdGrp\\Cbs\\Opn1\\Amt > 0 ) or ArgFxdGrp\\Cbs\\Opn2\\Amt <= 0 then
                    if (MdaUtils.compareTo(argFxdGrp.getCbs().getOpn1().getAmt(), 0) == 0) {
                        msgTxt = msgTxt + cr + errLB;
                        cr = CR;
                        res = false;
                    }
                }
                if (!MdaUtils.isEmpty(argFxdGrp.getRec().getClsdat())) {
                    msgTxt = msgTxt + cr + errClsY;
                    cr = CR;
                    res = false;
                }
                if (!checkRelStatus(argFxdGrp.getRec().getInr())) {
                    msgTxt = msgTxt + cr + errRel;
                    cr = CR;
                    res = false;
                }
                break;
            case "FXTATT":
                if (MdaUtils.isEmpty(argFxdGrp.getRec().getInr())) {
                    msgTxt = msgTxt + cr + errOpnN;
                    cr = CR;
                    res = false;
                }
                break;
        }
        // If $Res is set to false and no errormessage is set
        // set a default errormessage.
        if (!res && MdaUtils.isEmpty(msgTxt)) {
            msgTxt = I18nUtil.getMessage("funds_inffxd_000014");
        }
        // Transfer of assembled error message to ArgRes
//        argRes.value = msgTxt;
        trncodVo.setDisResult(msgTxt);
        trncodVo.setIsAllowed(res ? "Y" : "N");
    }

    /**
     * source:sysmod.0257.script
     */
    public boolean checkUsrBranch(String bchflg) {

        // 检查交易是否为允许的机构经办
        // 威海不区分资金部和清算部
        return true;

    }

    /**
     * source:fxdlod.0012.script
     */
    public boolean checkRelStatus(String objInr) {
        String sql = "SELECT count(inr) FROM TRN WHERE RELFLG='W' and INIFRM IN ('FXTSOP', 'FXTBOP', 'FXTSSS', 'FXTSSB', 'FXTSUS', 'FXTSUB') " +
                "AND INR IN (SELECT TRNINR FROM TRO WHERE OBJTYP = 'FXD' AND OBJINR='" + objInr + "' )";
        int count = publicMapper.dyncCount(ImmutableBiMap.of("sql", sql));
        return count <= 0;

    }

}
