package com.brilliance.isc.funds.crtcri.check;

import com.brilliance.isc.common.transaction.newcheck.AbstractModuleValidator;
import com.brilliance.isc.common.transaction.newcheck.ValidResult;
import com.brilliance.isc.common.transaction.newcheck.Validate;
import com.brilliance.isc.funds.bo.funds.CrtcriStoreBo;
import com.brilliance.mda.runtime.mda.util.MdaUtils;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.util.HashMap;
import java.util.Map;

/**
 * @program: isc-funds
 * @ClassName: CrtcriValidator
 * @description: 汇入清算 Crtcri 校验
 * @author: huangshunlin
 * @create: 2024-10-16 20:26
 */
@Component
public class CrtcriValidator extends AbstractModuleValidator<CrtcriStoreBo> {


    @Override
    @PostConstruct
    public void register() {
        super.register();
    }

    @Override
    public Map<String, String> validate(CrtcriStoreBo vo) {
        Map<String, String> maps = new HashMap<>();
        maps.putAll(super.validate(vo));
        return maps;
    }


    /**
     * source:crtcri.0006.script
     */
    @Validate(path = "crdgrp.rec.rcvobjtyp")
    public ValidResult checkCrdgrpRecRcvobjtypN1000(CrtcriStoreBo vo) {
        if (MdaUtils.isEmpty(vo.getCrdgrp().getRec().getRcvobjtyp())) {
            return errorMandatory();
        }
        return null;
    }

    /**
     * source:crtcri.0007.script
     */
    @Validate(path = "crdgrp.rec.rcvbchinr")
    public ValidResult checkCrdgrpRecRcvbchinrN1000(CrtcriStoreBo vo) {
        if (MdaUtils.isEmpty(vo.getCrdgrp().getRec().getRcvbchinr())) {
            return errorMandatory();
        }
        return null;
    }

    /**
     * source:crtcri.0015.script
     */
    @Validate(path = "crdgrp.cbs.max.amt")
    public ValidResult checkCrdgrpCbsMaxAmtN1000(CrtcriStoreBo vo) {
        if (MdaUtils.compareTo(vo.getCrdgrp().getCbs().getMax().getAmt(), 0) <= 0) {
            return ValidResult.build(false, getMessage("funds_fxtsqo_000002"));
        }
        return null;
    }

    /**
     * source:crtcri.0017.script
     */
    @Validate(path = "crdgrp.rec.msgref")
    public ValidResult checkCrdgrpRecMsgrefN1000(CrtcriStoreBo vo) {
        // 除了汇款交易，其他均要指定业务
        if (MdaUtils.compareTo(vo.getCrdgrp().getRec().getRcvobjtyp(), "CPD") != 0 && MdaUtils.isEmpty(vo.getCrdgrp().getRec().getRcvobjinr())) {
            return ValidResult.build(false, getMessage("funds_fxtsqo_000003"));
        }
        return null;

    }

    /**
     * source:crtcri.0023.script
     */
    @Validate(path = "crdgrp.rec.errmsg")
    public ValidResult checkCrdgrpRecErrmsgN1000(CrtcriStoreBo vo) {
        if (MdaUtils.compareTo(vo.getCrdgrp().getRec().getSta(), "E") == 0) {
            return ValidResult.build(false, getMessage("funds_fxtsqo_000004", vo.getCrdgrp().getRec().getErrmsg()));
        }
        return null;

    }


}
